@@ -6,7 +6,7 @@
 #
 # LLNL-CODE-797170
 # All rights reserved.
-# This file is part of Merlin, Version: 1.6.1.
+# This file is part of Merlin, Version: 1.7.0.
 #
 # For details, see https://github.com/LLNL/merlin.
 #
@@ -33,10 +33,7 @@ SampleIndex factory methods
 """
 from parse import parse
 
-from merlin.common.sample_index import (
-    MAX_SAMPLE,
-    SampleIndex,
-)
+from merlin.common.sample_index import MAX_SAMPLE, SampleIndex
 from merlin.utils import cd
 
 
