@@ -6,7 +6,7 @@
 #
 # LLNL-CODE-797170
 # All rights reserved.
-# This file is part of Merlin, Version: 1.6.1.
+# This file is part of Merlin, Version: 1.7.0.
 #
 # For details, see https://github.com/LLNL/merlin.
 #
@@ -128,15 +128,22 @@ def get_config(path):
     load_defaults(config)
     return config
 
-def load_default_timeout(config):
+
+def load_default_celery(config):
     try:
-        config["celery"]["visibility_timeout_seconds"]
+        config["celery"]
     except KeyError:
-        config["celery"]["visibility_timeout_seconds"] = 86400
+        config["celery"] = {}
+    try:
+        config["celery"]["override"]
+    except KeyError:
+        config["celery"]["override"] = None
+
 
 def load_defaults(config):
     load_default_user_names(config)
-    load_default_timeout(config)
+    load_default_celery(config)
+
 
 def is_debug():
     """
@@ -238,6 +245,7 @@ def get_ssl_entries(server_type, server_name, server_config, cert_path):
             "keyfile": "ssl_keyfile",
             "certfile": "ssl_certfile",
             "ca_certs": "ssl_ca_certs",
+            "cert_reqs": "ssl_cert_reqs",
         }
 
     # The mysql server requires key names with ssl_ and different var names
