@@ -6,7 +6,7 @@
 #
 # LLNL-CODE-797170
 # All rights reserved.
-# This file is part of Merlin, Version: 1.6.1.
+# This file is part of Merlin, Version: 1.7.0.
 #
 # For details, see https://github.com/LLNL/merlin.
 #
@@ -36,10 +36,7 @@ from __future__ import print_function
 import logging
 import os
 
-from merlin.config.configfile import (
-    CONFIG,
-    get_ssl_entries,
-)
+from merlin.config.configfile import CONFIG, get_ssl_entries
 
 
 try:
