@@ -6,7 +6,7 @@
 #
 # LLNL-CODE-797170
 # All rights reserved.
-# This file is part of Merlin, Version: 1.6.1.
+# This file is part of Merlin, Version: 1.7.0.
 #
 # For details, see https://github.com/LLNL/merlin.
 #
@@ -30,17 +30,13 @@
 
 import logging
 from collections import ChainMap
-from os.path import (
-    expanduser,
-    expandvars,
-)
+from copy import deepcopy
+from os.path import expanduser, expandvars
 
 from merlin.common.abstracts.enums import ReturnCode
-from merlin.spec.override import (
-    dump_with_overrides,
-    error_override_vars,
-)
+from merlin.spec.override import dump_with_overrides, error_override_vars
 from merlin.spec.specification import MerlinSpec
+from merlin.utils import contains_shell_ref, contains_token
 
 
 MAESTRO_RESERVED = {"SPECROOT", "WORKSPACE", "LAUNCHER"}
@@ -74,22 +70,28 @@ def var_ref(string):
     by $(<str>).
     """
     string = string.upper()
-    if "$(" in string:
+    if contains_token(string):
         LOG.warning(f"Bad var_ref usage on string '{string}'.")
         return string
     return f"$({string})"
 
 
-def expand_line(line, var_dict):
+def expand_line(line, var_dict, env_vars=False):
     """
-    Expand one line of text by substituting environment
-    and user variables, as well as variables in 'var_dict'.
+    Expand one line of text by substituting user variables, 
+    optionally environment variables, as well as variables in 'var_dict'.
     """
-    line = expandvars(expanduser(line))
-    if "$" not in line:
+    if (
+        (not contains_token(line))
+        and (not contains_shell_ref(line))
+        and ("~" not in line)
+    ):
         return line
     for key, val in var_dict.items():
-        line = line.replace(var_ref(key), str(val))
+        if key in line:
+            line = line.replace(var_ref(key), str(val))
+    if env_vars:
+        line = expandvars(expanduser(line))
     return line
 
 
@@ -98,6 +100,7 @@ def expand_by_line(text, var_dict):
     Given a text (yaml spec), and a dictionary of variable names
     and values, expand variables in the text line by line.
     """
+    text = text.splitlines()
     result = ""
     for line in text:
         expanded_line = expand_line(line, var_dict)
@@ -105,6 +108,33 @@ def expand_by_line(text, var_dict):
     return result
 
 
+def expand_env_vars(spec):
+    """
+    Expand environment variables for all sections of a spec, except
+    for values with the key 'cmd' or 'restart' (these are executable
+    shell scripts, so environment variable expansion would be redundant).
+    """
+
+    def recurse(section):
+        if section is None:
+            return section
+        if isinstance(section, str):
+            return expandvars(expanduser(section))
+        if isinstance(section, dict):
+            for k, v in section.items():
+                if k in ["cmd", "restart"]:
+                    continue
+                section[k] = recurse(v)
+        elif isinstance(section, list):
+            for i, elem in enumerate(deepcopy(section)):
+                section[i] = recurse(elem)
+        return section
+
+    for name, section in spec.sections.items():
+        setattr(spec, name, recurse(section))
+    return spec
+
+
 def determine_user_variables(*user_var_dicts):
     """
     Given an arbitrary number of dictionaries, determine them
@@ -133,15 +163,14 @@ def determine_user_variables(*user_var_dicts):
                 f"Cannot reassign value of reserved word '{key}'! Reserved words are: {RESERVED}."
             )
         new_val = str(val)
-        if "$(" in new_val:  # change to re
+        if contains_token(new_val):
             for determined_key in determined_results.keys():
                 var_determined_key = var_ref(determined_key)
                 if var_determined_key in new_val:
                     new_val = new_val.replace(
                         var_determined_key, determined_results[determined_key]
                     )
-        if "$" in new_val:  # change to re
-            new_val = expandvars(new_val)
+        new_val = expandvars(expanduser(new_val))
         determined_results[key.upper()] = new_val
     return determined_results
 
@@ -211,7 +240,7 @@ def expand_spec_no_study(filepath, override_vars=None):
         uvars.append(spec.environment["labels"])
     evaluated_uvars = determine_user_variables(*uvars)
 
-    return expand_by_line(full_spec.split("\n"), evaluated_uvars)
+    return expand_by_line(full_spec, evaluated_uvars)
 
 
 def get_spec_with_expansion(filepath, override_vars=None):
