@@ -6,7 +6,7 @@
 #
 # LLNL-CODE-797170
 # All rights reserved.
-# This file is part of Merlin, Version: 1.6.1.
+# This file is part of Merlin, Version: 1.7.0.
 #
 # For details, see https://github.com/LLNL/merlin.
 #
@@ -60,13 +60,13 @@ class Step:
         """
         get the run command text body"
         """
-        return self.mstep.step.to_dict()["run"]["cmd"]
+        return self.mstep.step.__dict__["run"]["cmd"]
 
     def get_restart_cmd(self):
         """
         get the restart command text body, else return None"
         """
-        return self.mstep.step.to_dict()["run"]["restart"]
+        return self.mstep.step.__dict__["run"]["restart"]
 
     def clone_changing_workspace_and_cmd(
         self, new_cmd=None, cmd_replacement_pairs=None, new_workspace=None
@@ -81,7 +81,7 @@ class Step:
         :param new_workspace : (Optional) the workspace for the new step.
         """
         LOG.debug(f"clone called with new_workspace {new_workspace}")
-        step_dict = deepcopy(self.mstep.step.to_dict())
+        step_dict = deepcopy(self.mstep.step.__dict__)
 
         if new_cmd is not None:
             step_dict["run"]["cmd"] = new_cmd
@@ -100,11 +100,15 @@ class Step:
         if new_workspace is None:
             new_workspace = self.get_workspace()
         LOG.debug(f"cloned step with workspace {new_workspace}")
-        return Step(_StepRecord(new_workspace, StudyStep.from_dict(step_dict)))
+        study_step = StudyStep()
+        study_step.name = step_dict["name"]
+        study_step.description = step_dict["description"]
+        study_step.run = step_dict["run"]
+        return Step(_StepRecord(new_workspace, study_step))
 
     def get_task_queue(self):
         """ Retrieve the task queue for the Step."""
-        return self.get_task_queue_from_dict(self.mstep.step.to_dict())
+        return self.get_task_queue_from_dict(self.mstep.step.__dict__)
 
     @staticmethod
     def get_task_queue_from_dict(step_dict):
@@ -121,7 +125,7 @@ class Step:
         """
         Returns the max number of retries for this step.
         """
-        return self.mstep.step.to_dict()["run"]["max_retries"]
+        return self.mstep.step.__dict__["run"]["max_retries"]
 
     def __get_restart(self):
         """
@@ -178,7 +182,7 @@ class Step:
         """
         :return : The step name.
         """
-        return self.mstep.step.to_dict()["name"]
+        return self.mstep.step.__dict__["name"]
 
     def execute(self, adapter_config):
         """
